// net.c - mythmon network routines
//
// Copyright (c) 2004 David George. ALL RIGHTS RESERVED.
// Copyright (c) 2007 David George. ALL RIGHTS RESERVED.
//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * You should have received a copy of this license with this source
 * code.  If you haven't you can view a copy at:
 *
 * http://www.gnu.org/licenses/gpl.txt
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 ****************************************************************************/

#include "mythmon.h"

int NetClientOpen(char *host, int port)
{
  struct hostent *phe;
  struct sockaddr_in rsin;
  int netfd;

  netfd = socket(AF_INET, SOCK_STREAM, 0);
  if (netfd == -1) {
    syslog(LOG_ERR, "NetClientOpen(%s:%d): socket() err=%d\n",
           host, port, errno);
    return(-1);
  }

  memset((void *)&rsin, 0, sizeof(rsin));
  if (is_num_address(host)) {
    rsin.sin_addr.s_addr = inet_addr(host);
  } else {
    if ((phe = gethostbyname(host)) == NULL) {
      syslog(LOG_ERR, "NetClientOpen(%s:%d): gethostbyname() err=%d\n",
             host, port, errno);
      close(netfd);
      return(-1);
    }
    memcpy((char *)&rsin.sin_addr, (char *)phe->h_addr_list[0],
           phe->h_length);
  }
  rsin.sin_family = AF_INET;
  rsin.sin_port = htons(port);

  if (connect(netfd, (struct sockaddr *)&rsin, sizeof(rsin)) == -1) {
    syslog(LOG_ERR, "NetClientOpen(%s:%d): connect() err=%d\n",
           host, port, errno);
    close(netfd);
    return(-1);
  }

  return(netfd);
}

int NetClose(int *netfd)
{
  if (*netfd != -1) {
    if (close(*netfd) == -1) {
      syslog(LOG_ERR, "NetClose: close(%d) err=%d\n", *netfd, errno);
      *netfd = -1;
      return(errno);
    }

    *netfd = -1;
  }

  return(0);
}

int NetRead(int netfd, const void *vptr, size_t n, uint32_t *ulRead)
{
  size_t nread;
  char *ptr;

  *ulRead = 0;
  ptr = (char *)vptr;
  nread = recv(netfd, ptr, n, 0);

  if (nread == 0)
    return(0);

  if (nread == -1) {
    if (errno == ECONNRESET)
      syslog(LOG_NOTICE, "NetRead: recv(%d) err=%d\n", netfd, errno);
    else
      syslog(LOG_ERR, "NetRead: recv(%d) err=%d\n", netfd, errno);

    return(-1);
  }

  *ulRead = (uint32_t)nread;
  return(0);
}

int NetReadTime(int netfd, char *str, int n, int init_time,
                int sec_time, uint32_t *ulRead)
{
  fd_set readset;
  struct timeval tval;
  int nread = 0, nfound, tread = 0;

  *ulRead = 0;
  FD_ZERO(&readset);
  tval.tv_sec = (int)(init_time / 1000);
  tval.tv_usec = (int)(init_time % 1000) * 1000;
  FD_SET(netfd, &readset);
  while (TRUE) {
    if (n - tread < 1)
      break;

    if (tval.tv_sec == 0 && tval.tv_usec == 0) {
      if ((nfound = select(netfd + 1, &readset, NULL, NULL, NULL)) < 1)
        break;
    } else {
      if ((nfound = select(netfd + 1, &readset, NULL, NULL, &tval)) < 1)
        break;
    }

    if (FD_ISSET(netfd, &readset)) {
      nread = recv(netfd, str+tread, n-tread, 0);
      if (nread == -1) {
        if (errno == ECONNRESET || errno == ESHUTDOWN)
          syslog(LOG_INFO, "NetReadTime: recv(%d) err=%d\n", netfd, errno);
        else
          syslog(LOG_ERR, "NetReadTime: recv(%d) err=%d\n", netfd, errno);

        return(-1);
      } else {
        if (nread == 0)
          return(-1);
        tread += nread;
      }
    }

    tval.tv_sec = (int)(sec_time / 1000);
    tval.tv_usec = (int)(sec_time % 1000) * 1000;
  }

  if (nfound == -1) {
    if (errno == ECONNRESET || errno == ESHUTDOWN)
      syslog(LOG_NOTICE, "NetReadTime: select(%d) err=%d\n", netfd, errno);
    else
      syslog(LOG_ERR, "NetReadTime: select(%d) err=%d\n", netfd, errno);

    return(-1);
  }

  *ulRead = (uint32_t)tread;
  return(0);
}

int NetWrite(int netfd, const void *vptr, size_t n, uint32_t *ulWritten)
{
  size_t nleft, nwritten;
  char *ptr;

  *ulWritten = 0;
  ptr = (char *)vptr;
  nleft = n;
  while (nleft > 0) {
    nwritten = send(netfd, ptr, nleft, 0);

    if (nwritten == 0)
      return(0);

    if (nwritten == -1) {
      if (errno == ECONNABORTED || errno == ECONNRESET)
        syslog(LOG_NOTICE, "NetWrite: send(%d) err=%d\n", netfd, errno);
      else
        syslog(LOG_ERR, "NetWrite: send(%d) err=%d\n", netfd, errno);

      return(-1);
    }

    nleft -= nwritten;
    ptr += nwritten;
    *ulWritten += (uint32_t)nwritten;
  }

  return(0);
}

int NetWriteString(int netfd, const void *vptr)
{
  size_t nleft, nwritten;
  char *ptr;

  ptr = (char *)vptr;
  nleft = strlen(vptr);

  while (nleft > 0) {
    nwritten = send(netfd, ptr, nleft, 0);

    if (nwritten == 0)
      return(0);

    if (nwritten == -1) {
      if (errno == ECONNABORTED || errno == ECONNRESET)
        syslog(LOG_NOTICE, "NetWrite: send(%d) err=%d\n",
               netfd, errno);
      else
        syslog(LOG_ERR, "NetWrite: send(%d) err=%d\n",
               netfd, errno);

      return(-1);
    }

    nleft -= nwritten;
    ptr += nwritten;
  }

  return(0);
}
